// ignore_for_file: public_member_api_docs, sort_constructors_first
// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

/// Calls on each cubit action of this package.
///
/// Useful to register custom logic on pre-implemented logic.
class CustomRoutine<R, Data> {
  const CustomRoutine({
    required this.routine,
    required this.attachedLogic,
    required this.onError,
    required this.onSuccess,
  });

  final FutureOr<Result<R, AppException>> Function() routine;
  final FutureOr<Result<Data?, AppException>> Function(
    Result<R, AppException> routineResult,
  ) attachedLogic;
  final void Function(AppException exception) onError;
  final void Function(R result, Data? data) onSuccess;

  FutureOr<void> call() async {
    final result = await routine.call();

    // Call custom logic
    final customRoutineResult = await attachedLogic.call(result);

    // Check for errors
    if (result.isErr) {
      onError.call(result.err!);

      return;
    }
    if (customRoutineResult.isErr) {
      onError.call(customRoutineResult.err!);

      return;
    }

    // If no error
    return onSuccess.call(result.ok as R, customRoutineResult.ok);
  }
}
