// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:firebase_auth/firebase_auth.dart';
import 'package:wyatt_authentication_bloc/src/core/exceptions/exceptions.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account.dart';

/// Account Model to parse Firebase User data
class AccountModel extends Account {
  factory AccountModel.fromFirebaseUserCredential(
    UserCredential? userCredential,
  ) {
    final user = userCredential?.user;
    if (user != null) {
      final providerId =
          (user.providerData.isEmpty) ? '' : user.providerData.first.providerId;
      return AccountModel._(
        user: user,
        id: user.uid,
        emailVerified: user.emailVerified,
        isAnonymous: user.isAnonymous,
        providerId: providerId,
        creationTime: user.metadata.creationTime,
        lastSignInTime: user.metadata.lastSignInTime,
        isNewUser: userCredential?.additionalUserInfo?.isNewUser,
        email: user.email,
        phoneNumber: user.phoneNumber,
        photoURL: user.photoURL,
        accessToken: userCredential?.credential?.accessToken,
      );
    } else {
      throw ModelParsingFailureFirebase('null-user', 'User cannot be null');
    }
  }

  factory AccountModel.fromFirebaseUser(User? user, {String? accessToken}) {
    if (user != null) {
      final providerId =
          (user.providerData.isEmpty) ? '' : user.providerData.first.providerId;
      return AccountModel._(
        user: user,
        id: user.uid,
        emailVerified: user.emailVerified,
        isAnonymous: user.isAnonymous,
        providerId: providerId,
        creationTime: user.metadata.creationTime,
        lastSignInTime: user.metadata.lastSignInTime,
        isNewUser: false,
        email: user.email,
        phoneNumber: user.phoneNumber,
        photoURL: user.photoURL,
        accessToken: accessToken,
      );
    } else {
      throw ModelParsingFailureFirebase('null-user', 'User cannot be null');
    }
  }
  const AccountModel._({
    required this.user,
    required super.id,
    required super.emailVerified,
    required super.isAnonymous,
    required super.providerId,
    super.lastSignInTime,
    super.creationTime,
    super.isNewUser,
    super.email,
    super.phoneNumber,
    super.photoURL,
    super.accessToken,
  });

  final User? user;

  @override
  String? get refreshToken => user?.refreshToken;
}
