// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/session_wrapper.dart';

/// Is responsible for abstracting the provenance of the data.
abstract class AuthenticationRemoteDataSource<Data>
    extends BaseRemoteDataSource {
  // Stream related methods ===================================================

  void addSession(SessionWrapper<Data> wrapper);

  Stream<SessionWrapper<Data>> sessionStream();

  // SignUp/SignIn methods ====================================================

  Future<Account> signUpWithEmailAndPassword({
    required String email,
    required String password,
  });

  Future<Account> signInWithEmailAndPassword({
    required String email,
    required String password,
  });
  Future<Account> signInAnonymously();
  Future<Account> signInWithGoogle();

  Future<void> signOut();

  // Account management methods ===============================================
  
  // Future<void> linkCurrentUserWith(AuthenticationProvider anotherProvider);
  Future<Account> refresh();
  Future<Account> reauthenticate();
  Future<Account> updateEmail({required String email});
  Future<Account> updatePassword({required String password});
  Future<void> delete();

  // Email related stuff ======================================================

  Future<void> sendEmailVerification();
  Future<void> sendPasswordResetEmail({required String email});
  Future<void> confirmPasswordReset({
    required String code,
    required String newPassword,
  });
  Future<bool> verifyPasswordResetCode({required String code});
}
